
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: _G, coroutine, next, pcall, string, type

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Integration LibSharedMedia")
local LSM = LibStub("LibSharedMedia-3.0", 1)

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.FontFilesToOverride = {
	["Fonts\\ARIALN.TTF"] = true,
	["Fonts\\FRIZQT__.TTF"] = true,
	["Fonts\\FRIZQT__.ttf"] = true,
	["Fonts\\MORPHEUS.ttf"] = true,
	["Fonts\\MORPHEUS.TTF"] = true,
	["Fonts\\skurri.ttf"] = true,
}

M.FontObjectToSize = {
	[SystemFont_LargeNamePlateFixed] = 20,
	[SystemFont_NamePlateCastBar] = 10,
	[SystemFont_NamePlateFixed] = 14,
	[SystemFont_World_ThickOutline] = 64,
	[SystemFont_World] = 64,
}

M.FontObjectsToUpdate = {
	[C.TEXT_LARGE] = true,
	[C.TEXT_MEDIUM] = true,
	[C.TEXT_REGULAR] = true,
}

function M:OnInitialize()
	self:RegisterFonts()
	self:RegisterSounds()
	self:RegisterStatusBars()
	self:SetSharedMediaFont(D.db.profile.font.appearance.name)
end

function M:OnEnable()
	self:RegisterMessage("SIXUI_CONFIG_UPDATE")
	self:RegisterMessage("SIXUI_PROFILE_UPDATE", "Update")
	self:Update()
end

function M:RegisterFonts()
	LSM:Register(LSM.MediaType.FONT, C.TEXT_FONT_NAME, C.TEXT_FONT_PATH)
end

function M:RegisterSounds()
	LSM:Register(LSM.MediaType.SOUND, "SixUI: 1", [[Interface\AddOns\SixUI\Resources\Words\1.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: 2", [[Interface\AddOns\SixUI\Resources\Words\2.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: 3", [[Interface\AddOns\SixUI\Resources\Words\3.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: 4", [[Interface\AddOns\SixUI\Resources\Words\4.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: 5", [[Interface\AddOns\SixUI\Resources\Words\5.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Affix", [[Interface\AddOns\SixUI\Resources\Words\Affix.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Alpha", [[Interface\AddOns\SixUI\Resources\Words\Alpha.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Assignment", [[Interface\AddOns\SixUI\Resources\Words\Assignment.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Assist", [[Interface\AddOns\SixUI\Resources\Words\Assist.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Attention", [[Interface\AddOns\SixUI\Resources\Words\Attention.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Beta", [[Interface\AddOns\SixUI\Resources\Words\Beta.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Boss", [[Interface\AddOns\SixUI\Resources\Words\Boss.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Circle of Healing", [[Interface\AddOns\SixUI\Resources\Words\Circle_of_Healing.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Covenant", [[Interface\AddOns\SixUI\Resources\Words\Covenant.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Danger", [[Interface\AddOns\SixUI\Resources\Words\Danger.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Dispel", [[Interface\AddOns\SixUI\Resources\Words\Dispel.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Divine Hymn", [[Interface\AddOns\SixUI\Resources\Words\Divine_Hymn.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: External", [[Interface\AddOns\SixUI\Resources\Words\External.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Focus", [[Interface\AddOns\SixUI\Resources\Words\Focus.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Group", [[Interface\AddOns\SixUI\Resources\Words\Group.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Marker", [[Interface\AddOns\SixUI\Resources\Words\Marker.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Minion", [[Interface\AddOns\SixUI\Resources\Words\Minion.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Omega", [[Interface\AddOns\SixUI\Resources\Words\Omega.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Party", [[Interface\AddOns\SixUI\Resources\Words\Party.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Prayer of Mending", [[Interface\AddOns\SixUI\Resources\Words\Prayer_of_Mending.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Racial", [[Interface\AddOns\SixUI\Resources\Words\Racial.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Raid", [[Interface\AddOns\SixUI\Resources\Words\Raid.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Ready", [[Interface\AddOns\SixUI\Resources\Words\Ready.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Rebuff", [[Interface\AddOns\SixUI\Resources\Words\Rebuff.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Safe", [[Interface\AddOns\SixUI\Resources\Words\Safe.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Salvation", [[Interface\AddOns\SixUI\Resources\Words\Salvation.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Shadow Word: Pain", [[Interface\AddOns\SixUI\Resources\Words\Shadow_Word_Pain.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Soak", [[Interface\AddOns\SixUI\Resources\Words\Soak.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Spirit Shell", [[Interface\AddOns\SixUI\Resources\Words\Spirit_Shell.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Spread", [[Interface\AddOns\SixUI\Resources\Words\Spread.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Stack", [[Interface\AddOns\SixUI\Resources\Words\Stack.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Summon", [[Interface\AddOns\SixUI\Resources\Words\Summon.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Target", [[Interface\AddOns\SixUI\Resources\Words\Target.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Trinket", [[Interface\AddOns\SixUI\Resources\Words\Trinket.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Yeet", [[Interface\AddOns\SixUI\Resources\Words\Yeet.ogg]])
	LSM:Register(LSM.MediaType.SOUND, "SixUI: Yikes", [[Interface\AddOns\SixUI\Resources\Words\Yikes.ogg]])
end

function M:RegisterStatusBars()
	LSM:Register(LSM.MediaType.STATUSBAR, "SixUI: Dark", [[Interface\AddOns\SixUI\Resources\Textures\Dark.tga]])
	LSM:Register(LSM.MediaType.STATUSBAR, "SixUI: Light", [[Interface\AddOns\SixUI\Resources\Textures\Light.tga]])
	LSM:Register(LSM.MediaType.STATUSBAR, "SixUI: Original", [[Interface\AddOns\SixUI\Resources\Textures\Original.tga]])
end

function M:Update()
	self:ResetSharedMediaFont()

	local config = D.db.profile.font.appearance
	local fontName = config.name
	local fontFile = LSM:Fetch(LSM.MediaType.FONT, fontName)

	self:SetFont(fontFile)

	if config.overrideGameFonts then
		self:SetGameFont(fontFile)
	else
		self:ResetGameFont()
	end

	if config.overrideSharedMediaFonts then
		self:SetSharedMediaFont(fontName)
	end
end

function M:IterateFonts(skipNewFonts)
	if not self.fontRegistry then
		self.fontRegistry = {}
	end

	if not skipNewFonts then
		for _, v in next, _G do
			if type(v) == "table" then
				local _, objectType = pcall(v.GetObjectType, v)

				if objectType == "Font" and v.GetFont and v.SetFont then
					local fontFile = v:GetFont()

					if not self.fontRegistry[v] and self.FontFilesToOverride[fontFile] and not self.FontObjectsToUpdate[v] then
						self.fontRegistry[v] = fontFile
					end
				end
			end
		end
	end

	return coroutine.wrap(function()
		for fontObject, originalFontFile in next, self.fontRegistry do
			coroutine.yield(fontObject, originalFontFile)
		end
	end)
end

function M:SetFont(fontFile)
	for fontObject in next, self.FontObjectsToUpdate do
		local _, fontSize, fontFlags = fontObject:GetFont()

		fontObject:SetFont(fontFile, fontSize, fontFlags)
	end
end

function M:SetGameFont(fontFile)
	for fontObject in self:IterateFonts() do
		local _, fontSize, fontFlags = fontObject:GetFont()
		local fixedSize = self.FontObjectToSize[fontObject]

		fontObject:SetFont(fontFile, fixedSize or fontSize, fontFlags)
	end
end

function M:ResetGameFont()
	for fontObject, originalFontFile in self:IterateFonts(true) do
		local _, fontSize, fontFlags = fontObject:GetFont()
		local fixedSize = self.FontObjectToSize[fontObject]

		fontObject:SetFont(originalFontFile, fixedSize or fontSize, fontFlags)
	end
end

function M:SetSharedMediaFont(fontName)
	LSM:SetGlobal(LSM.MediaType.FONT, fontName)
end

function M:ResetSharedMediaFont()
	LSM:SetGlobal(LSM.MediaType.FONT, nil)
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CONFIG_UPDATE(event, key, ...)
	local c, g, k = string.split(".", key)

	if c == "font" and g == "appearance" then
		self:Update()
	end
end
